@@ -552,22 +552,23 @@ int processtimeds()
 
 int processactions(struct timed *t)
 {
+	int delta;
+	int weight;
 	struct channel *this;
+
+	delta = mystate->now - mystate->last_action.ts;
 #ifdef DEBUG
 	/* prevent initial flooding of the console */
 	if((mystate->now - mystate->current_server->linkup) >= 10) {
 		oer_debug(OER_DEBUG_NOISE, "processactions(\"%lu\")\n", t->at);
-		oer_debug(OER_DEBUG_INFO, "processactions->last action at %lu / delta %d / barrier %d\n", mystate->last_action.ts, (int) (mystate->now - mystate->last_action.ts), mystate->last_action.barrier);
+		oer_debug(OER_DEBUG_INFO, "processactions->last action at %lu / delta %d / barrier %d\n", mystate->last_action.ts, delta, mystate->last_action.barrier);
 	}
 #endif
-	if(t->type != OER_TIMED_TYPE_KICK) {
-		if(t->prio == OER_TIMED_PRIORITY_CHANNEL_HANDLING ||
-		   t->prio == OER_TIMED_PRIORITY_CHANNEL_PROTECTION) {
-			if(mystate->now > mystate->last_action.ts || mystate->last_action.barrier < 5) {
-				mystate->last_action.ts = mystate->now;
-				mystate->last_action.barrier += 1 + (strlen(t->command) / 50);
-				return 1;
-			}
+	if((t->prio == OER_TIMED_PRIORITY_CHANNEL_HANDLING || t->prio == OER_TIMED_PRIORITY_CHANNEL_PROTECTION) && t->type != OER_TIMED_TYPE_KICK) {
+		if(delta || mystate->last_action.barrier < 5) {
+			mystate->last_action.ts = mystate->now;
+			mystate->last_action.barrier += 1 + (strlen(t->command) / 50);
+			return 1;
 		}
 	}
 	if(mystate->last_action.barrier) {
@@ -589,6 +590,13 @@ int processactions(struct timed *t)
 			mystate->last_action.barrier += 3 + (strlen(t->command) / 50);
 		}
 		break;
+	case OER_TIMED_PRIORITY_CTCP:
+		/* this is a flood protection, somebody could flood an otherwise
+		   idle oer by sending it ctcp queries in fixed intervals of 1..2
+		   seconds, it all comes down to ircd's flood detection */
+		weight = (delta) ? (2 / (delta + 1)) : 2;
+		mystate->last_action.barrier += weight + 2 + (strlen(t->command) / 50);
+		break;
 	case OER_TIMED_PRIORITY_NOTICE:
 	case OER_TIMED_PRIORITY_PRIVMSG:
 		mystate->last_action.barrier += 1 + (strlen(t->command) / 50);
